﻿Public Class Form1
    Dim CONNECTED As Boolean = False            '
    Dim readBuffer As String = ""               '
    Dim LEDSout As Integer = 0                  'variable that holds the value that gets written to the LEDs

    'flag variables to indicate if the specific button is currently ON or OFF
    Dim RED As Boolean = False
    Dim GREEN As Boolean = False



    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Fill up the cboBaud Combo box to common baud rates used
        cboBaud.Items.Add(9600)
        cboBaud.Items.Add(19200)
        cboBaud.Items.Add(38400)
        cboBaud.Items.Add(57600)
        cboBaud.Items.Add(115200)
        cboBaud.SelectedIndex = 4       'select 115200 as default

        ' Show all available COM ports in the combobox
        For Each sp As String In My.Computer.Ports.SerialPortNames
            cboComPort.Items.Add(sp)
        Next
        cboComPort.SelectedIndex = 0    'select first COM port in list as default

        Timer1.Interval = 100
        Timer1.Enabled = True
    End Sub

    Private Sub btnConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConnect.Click
        If Not CONNECTED Then   'if the COM port is not open yet, connect and open it.
            Try
                ' Try to open the selected COM port
                SerialPort1.PortName = cboComPort.Text
                SerialPort1.BaudRate = CInt(cboBaud.Text)
                SerialPort1.Open()
                cboComPort.BackColor = Color.LightGreen
                cboBaud.BackColor = Color.LightGreen
                CONNECTED = True
                btnConnect.Text = "Disconnect"
            Catch ex As Exception
                ' Cannot open the COM port
                MsgBox("Cannot open the COM port", MsgBoxStyle.Critical, "")
                cboComPort.BackColor = Color.White
                cboBaud.BackColor = Color.White
                CONNECTED = False
            End Try
        Else                    'if the COM port is open already, disconnect
            SerialPort1.Close()
            cboComPort.BackColor = Color.White
            cboBaud.BackColor = Color.White
            CONNECTED = False
            btnConnect.Text = "Connect"
        End If
    End Sub

    'send the value of LEDSout with cmd "00" to the STM32F3
    Sub SendLEDs()
        Dim temp As String

        'the protocol requires the message to be in HEX strings, must be in capital, and each field must have at 
        'least 2 characters, i.e. To set only the LEDs on Bit3-0 we cannot send "00f", it has to be "000F".
        temp = Hex(LEDSout).ToUpper     'converts an integer to a HEX string, e.g. 15 becomes "f".  ToUpper just makes sure everything is uppercase, i.e. "f" become "F"
        If temp.Length < 2 Then         'if the length of the converted string < 2, add a "0" to the beginning of the string				
            temp = "0" & temp
        End If

        lstTx.Items.Add("00" & temp)                'add the output message to lstTx
        lstTx.SelectedIndex = lstTx.Items.Count - 1 'highlight the last entry in the listbox lstTx
        SerialPort1.Write("00" & temp & vbLf)       'write the command "00" followed by the value of LEDSout as a HEX string (preceded by a "0" if required), followed by the LF character to terminate it
    End Sub

    Private Sub lblRed_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lblRed.Click
        If CONNECTED Then
            If RED Then
                RED = False
                lblRed.BackColor = Color.DarkGray
                LEDSout = LEDSout And Not (2)       'clear BIT2 in LEDSout without effecting any of the other bits
            Else
                RED = True
                lblRed.BackColor = Color.Red
                LEDSout = LEDSout Or 2              'set BIT2 in LEDSout without effecting any of the other bits
            End If
            SendLEDs()
        End If
    End Sub

    Private Sub lblGreen_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lblGreen.Click
        If CONNECTED Then
            If GREEN Then
                GREEN = False
                lblGreen.BackColor = Color.DarkGray
                LEDSout = LEDSout And Not (1)     'clear BIT0 in LEDSout without effecting any of the other bits
            Else
                GREEN = True
                lblGreen.BackColor = Color.Green
                LEDSout = LEDSout Or 1            'set BIT0 in LEDSout without effecting any of the other bits
            End If
            SendLEDs()
        End If
    End Sub

    'Clear all the LEDs when the user clicks on Reset
    Private Sub lblReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lblReset.Click
        If CONNECTED Then
            LEDSout = 0
            SendLEDs()

            lblRed.BackColor = Color.DarkGray
            lblGreen.BackColor = Color.DarkGray

            RED = False
            GREEN = False
        End If
    End Sub

    'Set all the LEDs when the user double-clicks on Reset
    Private Sub lblReset_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles lblReset.DoubleClick
        If CONNECTED Then
            LEDSout = 255
            SendLEDs()

            lblRed.BackColor = Color.Red
            lblGreen.BackColor = Color.Green

            RED = True
            GREEN = True
        End If
    End Sub

    'uses a label control to turn ON or OFF the flashing of the LEDs on the STM32F3
    Private Sub lblFlash_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lblFlash.Click
        If CONNECTED Then
            If lblFlash.Text = "OFF" Then                   'if toggle button was in OFF state
                lstTx.Items.Add("02")
                lstTx.SelectedIndex = lstTx.Items.Count - 1
                SerialPort1.Write("02" & vbLf)              'write the command "02" to turn ON flashing
                lblFlash.Text = "ON"
                lblFlash.BackColor = Color.LightGreen
            Else                                            'if toggle button was in ON state
                lstTx.Items.Add("03")
                lstTx.SelectedIndex = lstTx.Items.Count - 1
                SerialPort1.Write("03" & vbLf)              'write the command "03" to turn ON flashing
                lblFlash.Text = "OFF"
                lblFlash.BackColor = Color.DarkGray
            End If
        End If
    End Sub

    'send the value of the scroll bar with cmd "01" to the STM32F3
    Private Sub HScrollBar1_Scroll(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles HScrollBar1.Scroll
        Dim temp As String

        temp = Hex(HScrollBar1.Value).ToUpper           'Value is the current value of the seekbar.  In the Designer we set the max to 255, so Value can be between 0 and 255
        If temp.Length < 2 Then
            temp = "0" & temp
        End If
        lstTx.Items.Add("01" & temp)
        lstTx.SelectedIndex = lstTx.Items.Count - 1

        SerialPort1.Write("01" & temp & vbLf)           'write the command "01" followed by the value of the scrollbar as a HEX string (preceded by a "0" if required)
    End Sub

    'This event will be triggered every 100ms
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        If CONNECTED Then
            SerialPort1.Write("08" & vbLf)              'send the string "08" to the serialport.  Thsi will request the pushbutton and analog values from the STM32F3
        End If
    End Sub

    ' This event will be triggered when a string arrives on the serial port
    Private Sub SerialPort1_DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles SerialPort1.DataReceived
        Try
            readBuffer = SerialPort1.ReadLine()             'read the characters from the serial port
            'data to UI thread 
            Me.Invoke(New EventHandler(AddressOf DoUpdate)) 'update the received string
        Catch ex As Exception
            MsgBox("read " & ex.Message)
        End Try
    End Sub

    ' Update the received string and apply protocol calculations etc. here
    Public Sub DoUpdate(ByVal sender As Object, ByVal e As System.EventArgs)
        Dim data2, data1_0 As String
        Dim temp As Integer

        lstRx.Items.Add(readBuffer)
        lstRx.SelectedIndex = lstRx.Items.Count - 1

        'check if the reply from the STM32F3 starts with "09" to indicate 
        'that it Is replying To a request, e.g. "09010FFF".  Any other command will be ignored
        If readBuffer.StartsWith("09") Then
            data2 = readBuffer.Substring(2, 2)          'extract data2 from the message as a string (position 2 to 3), e.g. "01".
            data1_0 = readBuffer.Substring(4, 4)        'extract data1 and data0 as one string from the message (pos 4 to 7), e.g. "0FFF"
            If data2 = "01" Then
                lblButton.BackColor = Color.DarkBlue    'draw pushbutton ON when "01"
            Else
                lblButton.BackColor = Color.LightBlue   'draw pushbutton OFF when "00"
            End If
            temp = CInt("&h" & data1_0)                 'convert the hex string data1_0 to an integer
            hscAnalog.Value = CInt(temp)                'write the integer value to the scrollbar; The value is between 0 and 4095
        End If
    End Sub
End Class
